! code is taken from CESM1 and modified to interface with SAM
! Nov, 2018 Marat Khairoutdinov

SUBROUTINE oceflx (rbot, ubot  ,vbot,  tbot, qbot, thbot ,   & 
           &       zbot, ts, qs, wd, shf, lhf,   &
           &       taux  ,tauy  ,ra, raq, tref  ,qref  ,uref, vref )

   use grid, only: rank, dx, dy, mu, ady
   use params

   implicit none

! !INPUT/OUTPUT PARAMETERS:

   !--- input arguments --------------------------------
   real,intent(in) :: zbot  ! atm level height      (m)
   real,intent(in) :: ubot  ! atm u wind            (m/s)
   real,intent(in) :: vbot  ! atm v wind            (m/s)
   real,intent(in) :: thbot ! atm potential T       (K)
   real,intent(in) :: qbot  ! atm specific humidity (kg/kg)
   real,intent(in) :: rbot  ! atm air density       (kg/m^3)
   real,intent(in) :: tbot  ! atm T                 (K) 
   real,intent(in) :: ts    ! ocn temperature       (K)
   real,intent(in) :: qs    ! Surface saturation specific humidity(kg/kg)
   real,intent(in) :: wd    ! Wind enhansement -MK 2023

   !--- output arguments -------------------------------
   real,intent(out)  ::  shf   ! heat flux: sensible    (Km/s)
   real,intent(out)  ::  lhf   ! heat flux: latent      (m/s)
   real,intent(out)  ::  taux  ! surface stress, zonal      (N/m2)
   real,intent(out)  ::  tauy  ! surface stress, maridional (N/2)
   real,intent(out)  ::  ra    ! airodynamic resistence (heat)
   real,intent(out)  ::  raq   ! airodynamic resistence (vapor)
   real,intent(out)  ::  tref  ! diag:  2m ref height pot temp     (K)
   real,intent(out)  ::  qref  ! diag:  2m ref humidity (kg/kg)
   real,intent(out)  ::  uref  ! diag: 10m zonal wind (m/s)
   real,intent(out)  ::  vref  ! diag: 10m merid wind (m/s)
 
! !EOP

   !--- local constants --------------------------------
   real,parameter :: umin  =  0.5 ! minimum wind speed       (m/s)
   real,parameter :: zref  = 10.0 ! reference height           (m)
   real,parameter :: ztref =  2.0 ! reference height for air T (m)
   real,parameter :: karman = 0.4 ! 

   real,parameter :: missval=1.e30        
   !--- local variables --------------------------------
   real:: us=0.    ! ocn u-velocity        (m/s)
   real:: vs=0.    ! ocn v-velocity        (m/s)
   integer :: n      ! vector loop index
   real    :: vmag   ! surface wind magnitude   (m/s)
   real    :: thvbot ! virtual temperature      (K)
   real    :: delt   ! potential T difference   (K)
   real    :: delq   ! humidity difference      (kg/kg)
   real    :: stable ! stability factor
   real    :: rdn, rdn1    ! sqrt of neutral exchange coeff (momentum) 
   real    :: rhn, rhn1    ! sqrt of neutral exchange coeff (heat)     
   real    :: ren, ren1     ! sqrt of neutral exchange coeff (water)    
   real    :: rd     ! sqrt of exchange coefficient (momentum)         
   real    :: rh     ! sqrt of exchange coefficient (heat)             
   real    :: re     ! sqrt of exchange coefficient (water)            
   real    :: ustar  ! ustar             
   real    :: qstar  ! qstar             
   real    :: tstar  ! tstar             
   real    :: u10n   ! neutral 10 m wind speed over ocean
   real    :: hol    ! H (at zbot) over L
   real    :: xsq    ! ?
   real    :: xqq    ! ?
   real    :: psimh  ! stability function at zbot (momentum)
   real    :: psixh  ! stability function at zbot (heat and water)
   real    :: psix2  ! stability function at ztref reference height
   real    :: alz    ! ln(zbot/zref)
   real    :: al2    ! ln(zref/ztref)
   real    :: tau    ! stress at zbot
   real    :: bn     ! exchange coef funct for interpolation
   real    :: bh     ! exchange coef funct for interpolation
   real    :: fac    ! vertical interpolation factor
   real factor
   !--- local functions --------------------------------
   real    :: cdn    ! function: neutral drag coeff at 10m
   real    :: cdn_tc    ! function: neutral drag coeff at 10m for tropical cyclon
   real    :: psimhu ! function: unstable part of psimh
   real    :: psixhu ! function: unstable part of psimx
   real    :: Umps   ! dummy arg ~ wind velocity (m/s)
   real    :: Tk     ! dummy arg ~ temperature (K)
   real    :: xd     ! dummy arg ~ ?

   cdn(Umps)  =   0.0027 / Umps + 0.000142 + 0.0000764 * Umps
   cdn_tc(Umps)  = .0008 + .00004 * min(30.,Umps)
   psimhu(xd) = log((1.0+xd*(2.0+xd))*(1.0+xd*xd)/8.0) - 2.0*atan(xd) + 1.571
   psixhu(xd) = 2.0 * log((1.0 + xd*xd)/2.0)
 
!-------------------------------------------------------------------------------
! PURPOSE:
!   computes atm/ocn surface fluxes
!
! NOTES: 
!   o all fluxes are positive downward
!   o net heat flux = net sw + lw up + lw down + shf + lhf
!   o here, tstar = <WT>/U*, and qstar = <WQ>/U*.
!   o wind speeds should all be above a minimum speed (eg. 1.0 m/s)
! 
! ASSUMPTIONS:
!   o Neutral 10m drag coeff: cdn = .0027/U10 + .000142 + .0000764 U10
!   o Neutral 10m stanton number: ctn = .0327 sqrt(cdn), unstable
!                                 ctn = .0180 sqrt(cdn), stable
!   o Neutral 10m dalton number:  cen = .0346 sqrt(cdn)
!   o The saturation humidity of air at T(K): qsat(T)  (kg/m^3)

!  Tropical cyclon case (dotc=.true.) (Modified Large & Pond (1982) following Emanuel (2003,JAS):
!   1) Neutral 10m drag coeff:
!         cdn = .0008 + .00004 min(30.,U10N)
!   2) Neutral 10m stanton number:
!         ctn = cdn
!   3) Neutral 10m dalton number:
!         cen = cdn
!   in this code, those two approaches are merged, that is for large winds
!   it following the TC case
!   To make it on, the dotc parameter here should be set to .true.
!   Marat 2023
!-------------------------------------------------------------------------------

        al2 = log(zref/ztref)

        !--- compute some needed quantities ---
        vmag   = max(umin, sqrt( (ubot-us)**2 + (vbot-vs)**2 + wd**2)) 
        thvbot = thbot * (1.0 + epsv * qbot) ! virtual temp (K)
        delt   = thbot - ts                  ! pot temp diff (K)
        delq   = qbot - qs                     ! spec hum dif (kg/kg)
        alz    = log(zbot/zref) 
   
        !------------------------------------------------------------
        ! first estimate of Z/L and ustar, tstar and qstar
        !------------------------------------------------------------
   
        !--- neutral coefficients, z/L = 0.0 ---
        stable = 0.5 + sign(0.5 , delt)
        rdn    = sqrt(cdn(vmag))
        rhn    = (1.0-stable) * 0.0327 + stable * 0.018
        ren    = 0.0346
        if(dotc.and.vmag.gt.25.) then
         ! merge normal coeffs with hurrican based (wind in the range from 25 to 30 m/s):
           rdn1    = sqrt(cdn_tc(vmag))
           rhn1 = (1.0-stable)*rdn1 + stable * 0.018
           ren1 = rdn1
           factor = min(1.,(vmag-25.)*0.2)
           rdn = rdn + (rdn1-rdn)*factor
           rhn = rhn + (rhn1-rhn)*factor
           ren = ren + (ren1-ren)*factor
        end if

   
        !--- ustar, tstar, qstar ---
        ustar = rdn * vmag
        tstar = rhn * delt  
        qstar = ren * delq  
   
        !--- compute stability & evaluate all stability functions ---
        hol  = karman*ggr*zbot*  &
               (tstar/thbot+qstar/(1.0/epsv+qbot))/ustar**2
        hol  = sign( min(abs(hol),10.0), hol )
        stable = 0.5 + sign(0.5 , hol)
        xsq    = max(sqrt(abs(1.0 - 16.0*hol)) , 1.0)
        xqq    = sqrt(xsq)
        psimh  = -5.0*hol*stable + (1.0-stable)*psimhu(xqq)
        psixh  = -5.0*hol*stable + (1.0-stable)*psixhu(xqq)
   
        !--- shift wind speed using old coefficient ---
        rd   = rdn / (1.0 + rdn/karman*(alz-psimh))
        u10n = vmag * rd/rdn
   
        !--- update transfer coeffs at 10m and neutral stability ---
        rdn = sqrt(cdn(u10n))
        rhn = (1.0-stable)*0.0327 + stable * 0.018
        ren = 0.0346
        if(dotc.and.u10n.gt.25.) then
           rdn1    = sqrt(cdn_tc(u10n))
           rhn1 = (1.0-stable)*rdn1 + stable * 0.018
           ren1 = rdn1
           factor = min(1.,(u10n-25.)*0.2)
           rdn = rdn + (rdn1-rdn)*factor
           rhn = rhn + (rhn1-rhn)*factor
           ren = ren + (ren1-ren)*factor
        end if

    
        !--- shift all coeffs to measurement height and stability ---
        rd = rdn / (1.0 + rdn/karman*(alz-psimh)) 
        rh = rhn / (1.0 + rhn/karman*(alz-psixh)) 
        re = ren / (1.0 + ren/karman*(alz-psixh)) 
   
        !--- update ustar, tstar, qstar using updated, shifted coeffs --
        ustar = rd * vmag 
        tstar = rh * delt 
        qstar = re * delq 
    
        !------------------------------------------------------------
        ! iterate to converge on Z/L, ustar, tstar and qstar
        !------------------------------------------------------------
        !--- compute stability & evaluate all stability functions ---
        hol  = karman*ggr*zbot* &
               (tstar/thbot+qstar/(1.0/epsv+qbot))/ustar**2
        hol  = sign( min(abs(hol),10.0), hol )
        stable = 0.5 + sign(0.5 , hol)
        xsq    = max(sqrt(abs(1.0 - 16.0*hol)) , 1.0)
        xqq    = sqrt(xsq)
        psimh  = -5.0*hol*stable + (1.0-stable)*psimhu(xqq)
        psixh  = -5.0*hol*stable + (1.0-stable)*psixhu(xqq)
    
        !--- shift wind speed using old coeffs ---
        rd   = rdn / (1.0 + rdn/karman*(alz-psimh))
        u10n = vmag * rd / rdn 
        uref = ubot * rd / rdn 
        vref = vbot * rd / rdn 
    
        !--- update transfer coeffs at 10m and neutral stability ---
        rdn = sqrt(cdn(u10n))
        rhn = (1.0 - stable)*0.0327 + stable * 0.018
        ren = 0.0346
        if(dotc.and.u10n.gt.25.) then
           rdn1    = sqrt(cdn_tc(u10n))
           rhn1 = (1.0-stable)*rdn1 + stable * 0.018
           ren1 = rdn1
           factor = min(1.,(u10n-25.)*0.2)
           rdn = rdn + (rdn1-rdn)*factor
           rhn = rhn + (rhn1-rhn)*factor
           ren = ren + (ren1-ren)*factor
        end if

   
        !--- shift all coeffs to measurement height and stability ---
        rd = rdn / (1.0 + rdn/karman*(alz-psimh)) 
        rh = rhn / (1.0 + rhn/karman*(alz-psixh)) 
        re = ren / (1.0 + ren/karman*(alz-psixh)) 
    
        !--- update ustar, tstar, qstar using updated, shifted coeffs ---
        ustar = rd * vmag 
        tstar = rh * delt 
        qstar = re * delq 
    
        !------------------------------------------------------------
        ! compute the fluxes
        !------------------------------------------------------------
    
        tau = rbot * ustar * ustar 
       
        !--- momentum flux ---
        taux = -tau * (ubot-us) / vmag 
        tauy = -tau * (vbot-vs) / vmag 
        
        !--- heat flux ---
        shf  = -tau * tstar / ustar / rbot
        lhf  = -tau * qstar / ustar / rbot
      
        !--- water flux ---
    
        !------------------------------------------------------------
        ! compute diagnositcs: 2m ref T & Q, 10m wind speed squared
        !------------------------------------------------------------
        hol = hol*ztref/zbot
        xsq = max( 1.0, sqrt(abs(1.0-16.0*hol)) )
        xqq = sqrt(xsq)
        psix2   = -5.0*hol*stable + (1.0-stable)*psixhu(xqq)
        fac     = (rh/karman) * (alz + al2 - psixh + psix2 )
        tref = thbot - delt*fac 
        fac     = (re/karman) * (alz + al2 - psixh + psix2 )
        qref =  qbot - delq*fac
    
        ra = 1./(ustar*rh)
        raq = 1./(ustar*re)

END subroutine oceflx

